'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extractAndAssign = extractAndAssign;
exports.dynamicStyleStatement = dynamicStyleStatement;

var _state = require('../state');

var _state2 = _interopRequireDefault(_state);

var _helpers = require('./helpers');

var _hashSum = require('hash-sum');

var _hashSum2 = _interopRequireDefault(_hashSum);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// splits styles into static/dynamic pieces
function extractAndAssign(node, file) {
  // if array of objects
  if (_helpers.t.isArrayExpression(node.right)) {
    return getArrayStatics(node, file);
  }

  // extract statics, but return just dynamics
  if (_helpers.t.isObjectExpression(node.right)) {
    let name = node.left.name;

    if (_state2.default.viewStyleNames[name]) throw file.errorWithNode(node.left, `Duplicate style: view ${ _state2.default.inView } { ${ name } }`);

    _state2.default.viewStyleNames[name] = true;

    var _extractStatics = extractStatics(name, node.right, file);

    let statics = _extractStatics.statics;
    let dynamics = _extractStatics.dynamics;

    // sets dynamic keys for use in determining hot reload clear later

    const statKeys = _state2.default.viewStaticStyleKeys;
    const dynKeys = _state2.default.viewDynamicStyleKeys;

    statics.forEach(n => statKeys[n.key.name] = (0, _helpers.nodeToStr)(n.value));
    dynamics.forEach(n => {
      if (!_helpers.t.isSpreadProperty(n)) dynKeys[n.key.name] = (0, _helpers.nodeToStr)(n.value);
    });

    let hasStatics = statics.length;
    let hasDynamics = dynamics.length;

    let result = [];

    // if no dynamics, leave empty
    if (!hasStatics && !hasDynamics) return result;

    // hot reload uniq keys

    // keep statics hash inside view for child view styles (to trigger hot reloads)
    const isChildView = hasStatics && name[1] && name[1] == name[1].toUpperCase();
    const isChildViewClassed = hasStatics && _state2.default.viewHasChildWithClass && name != '$';

    if (isChildView || isChildViewClassed) {
      const uniq = (0, _hashSum2.default)(statKeys);
      result.push((0, _helpers.exprStatement)(_helpers.t.literal(uniq)));
    }

    // if dynamic + static clash, put that inside view to trigger hot reloads
    if (hasStatics && !_helpers.options.production && dynKeys.length) {
      let uniq = '';
      Object.keys(dynKeys).forEach(key => {
        if (statKeys[key]) {
          uniq += (0, _hashSum2.default)(statKeys[key] + dynKeys[key]) + (0, _hashSum2.default)(key);
        }
      });
      result.push((0, _helpers.exprStatement)(_helpers.t.literal(uniq)));
    }

    // return statement

    if (hasDynamics) {
      result.push(dynamicStyleStatement(node, dynamics));
    }

    return result;
  } else {
    return styleAssign(node);
  }
}

// find statics/dynamics in object
function extractStatics(name, node, file) {
  let statics = [];
  let dynamics = [];

  _state2.default.viewStyles[_state2.default.inView] = _state2.default.viewStyles[_state2.default.inView] || {};
  _state2.default.viewStyles[_state2.default.inView][name] = [];

  let duplicate = {};

  for (let prop of node.properties) {
    if (_helpers.t.isSpreadProperty(prop)) {
      // TODO: make work
      dynamics.push(prop);
      continue;
    }

    if (duplicate[prop.key.name]) throw file.errorWithNode(prop, `Duplicate style property: view ${ _state2.default.inView } { ${ name } = { ${ prop.key.name } } }`);

    duplicate[prop.key.name] = true;

    if (isStatic(prop)) {
      _state2.default.viewStyles[_state2.default.inView][name].push(prop);
      statics.push(prop);
    } else {
      dynamics.push(prop);
    }
  }

  return { statics, dynamics };
}

// determine if property is static
function isStatic(prop) {
  const staticKey = _helpers.t.isIdentifier(prop.key);

  if (!staticKey) return false;

  const staticVal = _helpers.t.isLiteral(prop.value) || _helpers.t.isUnaryExpression(prop.value) && _helpers.t.isLiteral(prop.value.argument);

  if (staticVal) return true;

  // determine if array is fully static
  if (_helpers.t.isArrayExpression(prop.value)) {
    return prop.value.elements.reduce((acc, cur) => acc = acc && _helpers.t.isLiteral(cur), true);
  }
}

// $["name"] = ...
function dynamicStyleStatement(node, dynamics) {
  return (0, _helpers.exprStatement)(styleAssign(node, _helpers.t.isArrayExpression(dynamics) ? dynamics : _helpers.t.objectExpression(dynamics)));
}

function styleLeft(node) {
  const prefix = '$';

  if (node.left.object) {
    return node.left;
  }

  const name = node.left.name.slice(1) || '$';
  return _helpers.t.identifier(`${ prefix }["${ name }"]`);
}

function styleAssign(node, _right) {
  let right = _right || node.right;

  const assignment = _helpers.t.assignmentExpression('=', styleLeft(node), styleFunction(right));

  assignment.isStyle = true;
  return assignment;

  // (_index) => {}
  function styleFunction(inner) {
    return _helpers.t.functionExpression(null, [_helpers.t.identifier('_'), _helpers.t.identifier('_index')], _helpers.t.blockStatement([_helpers.t.returnStatement(inner)]));
  }
}

// extract statics
function getArrayStatics(node, file) {
  let rightEls = [];
  let staticProps = [];

  let result = () => dynamicStyleStatement(node, node.right);

  for (let el of node.right.elements) {
    // bail out if they arent using just objects (ternery, variable)
    if (!_helpers.t.isObjectExpression(el)) {
      return result();
    }

    const extracted = extractStatics(node.left.name, el, file);
    if (!extracted) continue;

    let statics = extracted.statics;
    let dynamics = extracted.dynamics;


    if (statics.length) staticProps = staticProps.concat(statics);

    if (dynamics.length) {
      rightEls.push(_helpers.t.objectExpression(dynamics));
      continue;
    }
  }

  node.right.elements = rightEls;

  return result();
}
//# sourceMappingURL=extractStatics.js.map